@@ -92,6 +92,9 @@ func gopSpxTestExConf(t *testing.T, name string, conf *cl.Config, gmx, spxcode,
 		defer cl.SetDisableRecover(false)
 
 		fs := memfs.TwoFiles("/foo", spxfile, spxcode, gmxfile, gmx)
+		if gmxfile == "" {
+			fs = memfs.SingleFile("/foo", spxfile, spxcode)
+		}
 		pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", spxParserConf())
 		if err != nil {
 			scanner.PrintError(os.Stderr, err)
@@ -105,7 +108,7 @@ func gopSpxTestExConf(t *testing.T, name string, conf *cl.Config, gmx, spxcode,
 		var b bytes.Buffer
 		err = pkg.WriteTo(&b, resultFile)
 		if err != nil {
-			t.Fatal("gox.WriteTo failed:", err)
+			t.Fatal("gogen.WriteTo failed:", err)
 		}
 		result := b.String()
 		if result != expected {
@@ -135,6 +138,34 @@ func gopSpxErrorTestEx(t *testing.T, msg, gmx, spxcode, gmxfile, spxfile string)
 	}
 }
 
+func TestSpxNoGame(t *testing.T) {
+	gopSpxTestEx(t, ``, `
+`, `package main
+
+import "github.com/goplus/gop/cl/internal/spx"
+
+type Kai struct {
+	spx.Sprite
+	*MyGame
+}
+type MyGame struct {
+	*spx.MyGame
+}
+
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func (this *MyGame) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
+func main() {
+	new(MyGame).Main()
+}
+`, "", "Kai.tspx")
+}
+
 func TestSpxError(t *testing.T) {
 	gopSpxErrorTestEx(t, `Game.tgmx:6:2: userScore redeclared
 	Game.tgmx:5:2 other declaration of userScore`, `
@@ -197,8 +228,8 @@ func (this *Game) onInit() {
 func (this *Game) MainEntry() {
 	this.InitGameApp()
 }
-func main() {
-	spx.Gopt_MyGame_Main(new(Game))
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
 }
 func (this *Kai) onMsg(msg string) {
 	for {
@@ -206,6 +237,14 @@ func (this *Kai) onMsg(msg string) {
 		this.Say("Hi")
 	}
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -250,6 +289,90 @@ func (this *index) onInit() {
 	this.bar()
 	fmt.Println("Hi")
 }
+func (this *index) MainEntry() {
+}
+func (this *index) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
+func (this *bar) Classfname() string {
+	return "bar"
+}
+func (this *bar) Main() {
+}
+func main() {
+	new(index).Main()
+}
+`)
+}
+
+func TestEnvOp(t *testing.T) {
+	gopSpxTest(t, `
+echo ${PATH}, $id
+`, ``, `package main
+
+import (
+	"fmt"
+	"github.com/goplus/gop/cl/internal/spx"
+)
+
+type bar struct {
+	spx.Sprite
+	*index
+}
+type index struct {
+	*spx.MyGame
+}
+
+func (this *index) MainEntry() {
+	fmt.Println(this.Gop_Env("PATH"), this.Gop_Env("id"))
+}
+func (this *index) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
+func (this *bar) Classfname() string {
+	return "bar"
+}
+func (this *bar) Main() {
+}
+func main() {
+	new(index).Main()
+}
+`)
+}
+
+func TestSpxGopEnv(t *testing.T) {
+	gopSpxTest(t, `
+echo "${PATH}"
+`, ``, `package main
+
+import (
+	"fmt"
+	"github.com/goplus/gop/cl/internal/spx"
+	"strconv"
+)
+
+type bar struct {
+	spx.Sprite
+	*index
+}
+type index struct {
+	*spx.MyGame
+}
+
+func (this *index) MainEntry() {
+	fmt.Println(strconv.Itoa(this.Gop_Env("PATH")))
+}
+func (this *index) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
+func (this *bar) Classfname() string {
+	return "bar"
+}
+func (this *bar) Main() {
+}
+func main() {
+	new(index).Main()
+}
 `)
 }
 
@@ -283,8 +406,16 @@ func (this *index) MainEntry() {
 		this.Gop_Exec("ls", "-l")
 	})
 }
+func (this *index) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
+func (this *bar) Classfname() string {
+	return "bar"
+}
+func (this *bar) Main() {
+}
 func main() {
-	spx.Gopt_MyGame_Main(new(index))
+	new(index).Main()
 }
 `)
 }
@@ -325,12 +456,25 @@ func (this *Game) onInit() {
 	spx.TestIntValue = 1
 	x := math.Round(1.2)
 }
+func (this *Game) MainEntry() {
+}
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
 func (this *bar) onInit() {
 	this.SetCostume("kai-a")
 	this.Play("recordingWhere")
 	this.Say("Where do you come from?", 2)
 	this.Broadcast__0("msg2")
 }
+func (this *bar) Classfname() string {
+	return "bar"
+}
+func (this *bar) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "bar.tspx")
 }
 
@@ -374,12 +518,25 @@ func (this *Game) onInit() {
 	spx.Gopt_Sprite_Clone__0(this.Kai)
 	this.Broadcast__0("msg1")
 }
+func (this *Game) MainEntry() {
+}
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
 func (this *Kai) onInit() {
 	this.a = 1
 }
 func (this *Kai) onCloned() {
 	this.Say("Hi")
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -417,12 +574,18 @@ var x float64 = spx.Rand__1(1.2)
 func (this *index) MainEntry() {
 	spx.Gopt_MyGame_Run(this, "hzip://open.qiniu.us/weather/res.zip")
 }
-func main() {
-	spx.Gopt_MyGame_Main(new(index))
+func (this *index) Main() {
+	spx.Gopt_MyGame_Main(this)
 }
 func (this *Kai) Main() {
 	fmt.Println("Hi")
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func main() {
+	new(index).Main()
+}
 `, "index.tgmx", "Kai.tspx")
 }
 
@@ -455,12 +618,19 @@ type Game struct {
 func (this *Game) MainEntry() {
 	this.Run()
 }
-func main() {
-	spx3.Gopt_Game_Main(new(Game), new(Kai))
+func (this *Game) Main() {
+	spx3.Gopt_Game_Main(this, new(Kai))
 }
-func (this *Kai) Main() {
+func (this *Kai) Main(_gop_arg0 string) {
+	this.Sprite.Main(_gop_arg0)
 	fmt.Println(jwt.Token("Hi"))
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func main() {
+	new(Game).Main()
+}
 `, "main_spx.gox", "Kai_spx.gox")
 }
 
@@ -491,8 +661,17 @@ func (this *Game) MainEntry() {
 	b := new(spx3.Sprite)
 	fmt.Println(b.Name())
 }
+func (this *Game) Main() {
+	spx3.Gopt_Game_Main(this, new(Kai))
+}
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main(_gop_arg0 string) {
+	this.Sprite.Main(_gop_arg0)
+}
 func main() {
-	spx3.Gopt_Game_Main(new(Game), new(Kai))
+	new(Game).Main()
 }
 `, "main_spx.gox", "Kai_spx.gox")
 }
@@ -521,11 +700,19 @@ type Kai struct {
 func (this *Game) MainEntry() {
 	fmt.Println("Hi")
 }
-func main() {
-	new(Game).Main()
+func (this *Game) Main() {
+	(*spx2.Game).Main(&this.Game)
 }
 func (this *Kai) onMsg(msg string) {
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.t2gmx", "Kai.t2spx")
 }
 
@@ -553,11 +740,19 @@ type Kai struct {
 func (this *Game) MainEntry() {
 	fmt.Println("Hi, Sprite2")
 }
-func main() {
-	new(Game).Main()
+func (this *Game) Main() {
+	(*spx2.Game).Main(&this.Game)
 }
 func (this *Kai) onMsg(msg string) {
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.t2gmx", "Kai.t2spx2")
 }
 
@@ -584,8 +779,16 @@ type Kai struct {
 func (this *Dog) Main() {
 	fmt.Println("Hi, Sprite")
 }
+func (this *Dog) Classfname() string {
+	return "Dog"
+}
 func (this *Kai) onMsg(msg string) {
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
 `, "Dog_t3spx.gox", "Kai.t3spx2")
 }
 
@@ -610,6 +813,14 @@ type Kai struct {
 
 func (this *Game) MainEntry() {
 }
+func (this *Game) Main() {
+	(*spx2.Game).Main(&this.Game)
+}
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
 func main() {
 	new(Game).Main()
 }
@@ -634,6 +845,14 @@ type Kai struct {
 
 func (this *Game) MainEntry() {
 }
+func (this *Game) Main() {
+	(*spx2.Game).Main(&this.Game)
+}
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
 func main() {
 	new(Game).Main()
 }
@@ -671,14 +890,20 @@ type Kai struct {
 func (this *Game) MainEntry() {
 	fmt.Println("Hi")
 }
-func main() {
-	new(Game).Main()
+func (this *Game) Main() {
+	(*spx2.Game).Main(&this.Game)
 }
 func (this *Kai) Main() {
 	fmt.Println("Hello")
 }
 func (this *Kai) onMsg(msg string) {
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.t2gmx", "Kai.t2spx", "")
 }
 
@@ -711,12 +936,25 @@ func (this *Game) onInit() {
 		spx.SchedNow()
 	}
 }
+func (this *Game) MainEntry() {
+}
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
 func (this *Kai) onMsg(msg string) {
 	for {
 		spx.Sched()
 		this.Say("Hi")
 	}
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -772,6 +1010,11 @@ func (this *Game) onInit() {
 	spx.Gopt_Sprite_Clone__0(this.Kai)
 	this.Broadcast__0("msg1")
 }
+func (this *Game) MainEntry() {
+}
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
 func (this *Kai) onInit() {
 	this.a = 1
 	spx.Gopt_Sprite_Clone__0(this)
@@ -781,6 +1024,14 @@ func (this *Kai) onInit() {
 func (this *Kai) onCloned() {
 	this.Say("Hi")
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -813,11 +1064,19 @@ type Kai struct {
 func (this *Game) MainEntry() {
 	this.SendMessage("Hi")
 }
-func main() {
-	spx.Gopt_MyGame_Main(new(Game))
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
 }
 func (this *Kai) onMsg(msg string) {
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -846,12 +1105,20 @@ type Kai struct {
 func (this *Game) MainEntry() {
 	fmt.Println("Hi")
 }
-func main() {
-	spx.Gopt_MyGame_Main(new(Game))
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
 }
 func (this *Kai) onMsg(msg string) {
 	this.Position().Add__0(100, 200)
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -893,8 +1160,8 @@ type Kai struct {
 func (this *Game) MainEntry() {
 	fmt.Println("hi")
 }
-func main() {
-	spx.Gopt_MyGame_Main(new(Game))
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
 }
 func (this *Kai) onMsg(msg string) {
 	fmt.Println(msg)
@@ -911,6 +1178,14 @@ func (this *Kai) onMsg(msg string) {
 	fmt.Println(this.Vector().X)
 	fmt.Println(this.Vector().Self().Self())
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func (this *Kai) Main() {
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -982,6 +1257,11 @@ func (this *Game) onInit() {
 	spx.Gopt_Sprite_OnKey__1(this.Kai, "hello", func(key string) {
 	})
 }
+func (this *Game) MainEntry() {
+}
+func (this *Game) Main() {
+	spx.Gopt_MyGame_Main(this)
+}
 func (p *Mesh) Name() string {
 	return "hello"
 }
@@ -1012,6 +1292,12 @@ func (this *Kai) Main() {
 	spx.Gopt_Sprite_OnKey2(this, "hello", func(key string) {
 	})
 }
+func (this *Kai) Classfname() string {
+	return "Kai"
+}
+func main() {
+	new(Game).Main()
+}
 `, "Game.tgmx", "Kai.tspx")
 }
 
@@ -1048,6 +1334,9 @@ func (this *caseFoo) Main() {
 		t.Fatal("failed")
 	})
 }
+func (this *caseFoo) Classfname() string {
+	return "Foo"
+}
 func TestFoo(t *testing.T) {
 	test.Gopt_Case_TestMain(new(caseFoo), t)
 }
@@ -1076,6 +1365,9 @@ type case_foo struct {
 func (this *case_foo) Main() {
 	this.T().Log("Hi")
 }
+func (this *case_foo) Classfname() string {
+	return "foo"
+}
 func Test_foo(t *testing.T) {
 	test.Gopt_Case_TestMain(new(case_foo), t)
 }
