@@ -23,11 +23,11 @@ import (
 	"os"
 	"reflect"
 
+	"github.com/goplus/gogen"
 	"github.com/goplus/gop"
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/cmd/internal/base"
 	"github.com/goplus/gop/x/gopprojs"
-	"github.com/goplus/gox"
 	"github.com/qiniu/x/errors"
 )
 
@@ -66,7 +66,7 @@ func runCmd(cmd *base.Command, args []string) {
 	}
 
 	if *flagVerbose {
-		gox.SetDebug(gox.DbgFlagAll &^ gox.DbgFlagComments)
+		gogen.SetDebug(gogen.DbgFlagAll &^ gogen.DbgFlagComments)
 		cl.SetDebug(cl.DbgFlagAll)
 		cl.SetDisableRecover(true)
 	}
