@@ -1067,7 +1067,7 @@ func (p *printer) expr1(expr ast.Expr, prec1, depth int) {
 			p.expr(x.Lhs[0])
 			p.print(blank)
 		}
-		p.print(token.RARROW, blank)
+		p.print(token.DRARROW, blank)
 		if x.RhsHasParen {
 			p.print(token.LPAREN)
 			p.exprList(token.NoPos, x.Rhs, 1, noIndent, token.NoPos, false)
@@ -1085,7 +1085,7 @@ func (p *printer) expr1(expr ast.Expr, prec1, depth int) {
 			p.expr(x.Lhs[0])
 			p.print(blank)
 		}
-		p.print(token.RARROW, blank)
+		p.print(token.DRARROW, blank)
 		p.block(x.Body, 1)
 
 	case *ast.RangeExpr:
@@ -1101,6 +1101,14 @@ func (p *printer) expr1(expr ast.Expr, prec1, depth int) {
 			p.expr(x.Expr3)
 		}
 
+	case *ast.EnvExpr:
+		p.print(token.ENV)
+		if x.HasBrace() {
+			p.print(token.LBRACE, x.Name, token.RBRACE)
+		} else {
+			p.print(x.Name)
+		}
+
 	default:
 		log.Fatalf("unreachable %T\n", x)
 	}
